%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Name:  MSSegPois.m	Created:  01/09/06
%
% Usage: Compute multiscale segmentation of a Poisson time series,
%        based on the material in the paper
%		Kolaczyk, E.D. and Nowak, R.D. (2000).  
%		A multiresolution analysis for likelihoods:
%		theory and methods.  Submitted.
%
% Inputs: x := vector of independent Poisson counts (n x 1),
%	       where n need not be a power of 2.
% Outputs: bp := vector with locations of segmentation (break) points,
%		 with elements between 1 and n.
%          est := estimate of intensity underlying the Poisson count
%		  data, created by averaging over each segment in the
%		  segmentation. (n x 1)
% Calls:  TriangleAlgPois.m, GetBreakpoints.m
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [bp, est] = MSSegPois(x)

n = length(x);

%% Algorithm consists of two steps:
%   (i)  Computing optimal local (in scale and position) probabilities
%        for intervals of all sizes 1 to n.  This may be done in a
%        recursive, pyramidal fashion and stored in a set of 
%        triangular arrays.
%   (ii) Extracting from the information in the arrays the segmentation
%        that is actually optimal.

% Run triangle algorithm.
[t1, t2, m2] = TriangleAlgPois(x);

% Extract breakpoints.
bp = [];
bp = GetBreakpoints(bp,1,n,t1,t2,m2);
bp = sort(bp);

% Compute piecewise constant estimator from data and breakpoint info.
bp = [0 bp];
if bp == 0,
	est = mean(x)*ones(size(x));
else
for i=2:length(bp)
  est(bp(i-1)+1:bp(i)) = mean(x(bp(i-1)+1:bp(i)))*...
			   ones(size(x(bp(i-1)+1:bp(i))));
end
est(bp(i)+1:length(x)) = mean(x(bp(i)+1:length(x)))*...
                           ones(size(x(bp(i)+1:length(x))));
end

%  This code has been developed by Eric Kolaczyk and Robert Nowak.
%  Copyright (c) Eric Kolaczyk and Robert Nowak, 2001.
%  All rights reserved.
