%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Name:  TriangleAlgPois.m    Created:  00/07/24   Revised: 01/09/06
%
% Usage: Performs a recursive, pyramidal series of local (in scale 
%        and position) probability optimizations.  Essentially,
%        (log) probabilities for the two hypotheses of "homogeneous" or
%        "inhomogeneous" are computed for all substrings of the data
%        vector, starting with length 1 and building recursively to
%        the full length n.  These are stored in two separate triangular
%        arrays.  Each row of the arrays corresponds to substrings (or 'blocks')
%        of data of length/scale 1,2,...,n.  Additionally, a mask is built 
%        along the way to indicate the positions of 'split' decisions at each
%        scale.
%
% Inputs: x := vector of independent Poisson counts (n x 1),
%              where n need not be a power of 2.
% Outputs: t0 := triangular array of probability values under null
%                hypothesis (i.e., homogeneity)
%          t1 := triangular array of probability values under alternative
%                hypothesis (i.e., inhomogeneity).
%          mm := mask indicating (with a '1') the decided split 
%		 locations in the sequence (in relation to the  
%                arrays t0, t1).
%
% Calls:  log_bi_calc.m
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [t0,t1,mm] = TriangleAlgPois(x);

% Some preliminary stuff.
clear t0
clear t1

n = length(x);

for i=1:n
     t0(i,i) = 1;
     t1(i,i) = 1;
     mm(i,i) = 0;
end

% Begin iterative calculation of probabilities over intervals of 
% increasing size, storing the various results in the upper triangles
% of the matrices t0, t1, and mm.
for j=1:n-1
  for i=1:n-j
    p0 = log_bi_calc(x,i,i+j)+t0(i,i)+t0(i+1,i+j)';
    p1 = -log(sum(x(i:i+j))+1)+t1(i,i-1+(1:j))+...
		t1(i+(1:j),i+j)';
    t0(i,i+j) = p0;
    t1(i,i+j) = max(max(p1),t0(i,i+j));
    [dummy,mm(i,i+j)] = max(p1);
    mm(i,i+j) = mm(i,i+j)+i-1;
  end
end

%  This code has been developed by Eric Kolaczyk and Robert Nowak.
%  Copyright (c) Eric Kolaczyk and Robert Nowak, 2000.
%  All rights reserved.
